/*
FILE:     htmlUtil.C
*/
/*
VERSION:  2.250
*/
/*
DATE:     10/21/2013
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2013 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/*
 *  Collection of utilities for generating mmcif -> html
 */ 



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <string>

#include "htmlUtil.h"
#include "CifString.h"

using std::string;

static int  dq = '"';

/* 
 * Global string pointing to the current dictionary... 
 * Used to select the path 
 */

extern char *DICTIONARY_VERSION;

void htput_title(FILE *fp,const char *title)
{
  fprintf(fp,"<TITLE> %s </TITLE>\n",title);
}


void htput_open_body(FILE *fp, const char *title)
{
#if defined(HTMLSSI)
  fprintf(fp,"<!DOCTYPE HTML PUBLIC %c-//W3C//DTD HTML 4.01//EN%c>\n",dq,dq);
  fprintf(fp,"<HTML>\n");
  fprintf(fp,"<HEAD>\n");
  htput_title(fp,title);
  fprintf(fp,"<meta http-equiv=%cContent-Type%c content=%ctext/html; charset=iso-8859-1%c>\n",dq,dq,dq,dq);
  fprintf(fp,"<LINK HREF=%c/styles/general.css%c REL=%cSTYLESHEET%c TYPE=%ctext/css%c>\n",dq,dq,dq,dq,dq,dq);
  fprintf(fp,"</HEAD>\n");
  fprintf(fp,"<BODY>\n");
  fprintf(fp,"<!--#include virtual=%c../html-includes/header.txt%c-->\n",dq,dq);
#else
  fprintf(fp,"<HTML>\n");
  fprintf(fp,"<HEAD>\n");
  htput_title(fp,title);
  fprintf(fp,"<LINK HREF=%c/styles/general.css%c REL=%cSTYLESHEET%c TYPE=%ctext/css%c>\n",dq,dq,dq,dq,dq,dq);
  fprintf(fp,"</HEAD>\n");
  fprintf(fp,"<BODY>\n");
#endif
}


void htput_open_body_1(FILE *fp, const char *title)
{
  fprintf(fp,"<!DOCTYPE HTML PUBLIC %c-//W3O//DTD W3 HTML 2.0//EN%c>\n",dq,dq);
  fprintf(fp,"<HTML>\n");
  fprintf(fp,"<HEAD>\n");
  htput_title(fp,title);
  fprintf(fp,"</HEAD>\n");
  fprintf(fp,"<BODY bgcolor=%c%s%c >\n", dq,HT_BG_COLOR,dq);
  fprintf(fp,"<BASEFONT SIZE=%d >\n\n",HT_BASE_FONT_SIZE);
}


void htput_close_body(FILE *fp)
{

#if defined(HTMLSSI)
  fprintf(fp,"<!--#include virtual=%c../html-includes/footer.txt%c-->\n",dq,dq);
#endif

  fprintf(fp,"</BODY>\n");
  fprintf(fp,"</HTML>\n");
}
 
void htput_separator(FILE *fp)
{
#if defined(HTMLSSI)
  fprintf(fp,"\n");
#else
  htput_open_center(fp);
  fprintf(fp,"<IMG SRC=%c../Icons/%s%c ALIGN=MIDDLE>\n",
	  dq,
	  HT_IMAGE_BAR,
	  dq);
  htput_close_center(fp);
#endif
}

void htput_ndb_address(FILE *fp)
{
#if defined(HTMLSSI)
  fprintf(fp,"\n");
#else
  fprintf(fp,"<BR><BR>\n");
  htput_separator(fp);
  htput_open_center(fp);
  fprintf(fp,"<BR>\n");
  fprintf(fp,"<FONT SIZE=2>\n");
  fprintf(fp,"This HTML page was created using the\n");
  fprintf(fp," <A HREF=%chttp://sw-tools.pdb.org/apps/CIFOBJ%c> CIFOBJ </A> dictionary access tool set\n",dq,dq);
  fprintf(fp,"<BR>\n");
  fprintf(fp," at the\n");              
  fprintf(fp,"<ADDRESS>\n");
  fprintf(fp,"Resource Collaboratory for Structural Bioinformatics <BR>\n");
  fprintf(fp,"Rutgers University, Department of Chemistry<BR>\n");
  fprintf(fp,"New Brunswick, New Jersey<BR>\n");
  fprintf(fp," <A HREF=%cmailto:help@rcsb.rutgers.edu%c> help@rcsb.rutgers.edu </A>\n",dq,dq);
  fprintf(fp,"</ADDRESS>\n");
  fprintf(fp,"</FONT>\n");
  htput_close_center(fp);
#endif
}


void htput_header1_text(FILE *fp,const char *text)
{
  htput_open_center(fp);
  fprintf(fp,"<P><H1>%s</H1>\n",text);
  htput_close_center(fp);
}


void htput_header1_text_with_icon(FILE *fp, const char *text,  const char *icon)
{
  fprintf(fp,"<br><h1><IMG SRC=%c../Icons/%s%c ALIGN=TOP>  %s </h1>\n",
	  dq,
	  icon,
	  dq,text);
}


void htput_header2_text(FILE *fp, const char *text)
{
  htput_open_center(fp);
  fprintf(fp,"<br><H2>%s</H2>\n",text);
  htput_close_center(fp);
}

void htput_header2_text_with_icon(FILE *fp, const char *text,  const char *icon)
{
  fprintf(fp,"<br><h2><IMG SRC=%c../Icons/%s%c ALIGN=TOP> %s </h2>\n",
	  dq,
	  icon,
	  dq,text);
}



void htput_header3_text(FILE *fp,const char *text)
{
  htput_open_center(fp);
  fprintf(fp,"<br><H3>%s</H3>\n",text);
  htput_close_center(fp);
}


void htput_header3_text_with_icon(FILE *fp, const char *text,  const char *icon)
{
  fprintf(fp,"<br><h3><IMG SRC=%c../Icons/%s%c ALIGN=TOP>  %s </h3>\n",
	  dq,
	  icon,
	  dq,text);
}


void htput_paragraph_break(FILE *fp)
{
  fprintf(fp,"<P>\n");
}

void htput_line_break(FILE *fp)
{
  fprintf(fp,"<BR>\n");
}

void htput_text(FILE *fp,const char *text)
{
  fprintf(fp,"%s\n",text);
}

void htput_centered_text(FILE *fp,const char *text)
{
  htput_open_center(fp);  
  fprintf(fp,"%s\n",text);
  htput_close_center(fp);
}


void htput_centered_menu_text(FILE *fp,const char *name)
{
  fprintf(fp,"<BR>\n");
  htput_open_center(fp);
  //  fprintf(fp,"<FONT SIZE=%d> %s </FONT>",HT_MENU_NAME_FONT_SIZE,name);
  fprintf(fp,"%s",name);
  htput_close_center(fp);
}

void htput_centered_label(FILE *fp,const char *label)
{
  fprintf(fp,"<BR>\n");
  htput_open_center(fp);
  //  fprintf(fp,"<FONT SIZE=%d> %s </FONT>",HT_LABEL_FONT_SIZE,label);
  fprintf(fp,"%s",label);
  htput_close_center(fp);
}

#if 0
void htput_centered_menu_file_item(FILE *fp,const char *file, const char *item)
{
  htput_open_center(fp);
  fprintf(fp,"<P><A HREF=%c%s/%s%c><FONT SIZE=%d> %s </FONT></A>",
	 dq,
	 HT_SERVER_NAME,
	 file,
	 dq,HT_MENU_ITEM_FONT_SIZE,item);
  htput_close_center(fp);
}
#endif


void htput_open_table(FILE *fp)
{
  fprintf(fp,"<TABLE>\n");
}

void htput_open_table_with_border(FILE *fp)
{
  fprintf(fp,"<TABLE BORDER>\n");
}


void htput_close_table(FILE *fp)
{
  fprintf(fp,"</TABLE>\n");
}

void htput_open_row(FILE *fp)
{
  fprintf(fp,"<TR>\n");
}


void htput_close_row(FILE *fp)
{
  fprintf(fp,"</TR>\n");
}



void htput_table_col_header(FILE *fp, const char *text)
{
  //  fprintf(fp,"<TH VALIGN=TOP ALIGN=CENTER NOWRAP><FONT SIZE=%d> %s </FONT></TH>\n",
  //              HT_TABLE_HEADER_FONT_SIZE,text);
  fprintf(fp,"<TH VALIGN=TOP ALIGN=CENTER NOWRAP>%s</TH>\n",text);



}

void htput_table_col_text(FILE *fp, const char *text)
{
  //  fprintf(fp,"<TD ALIGN=CENTER><FONT SIZE=%d> %s </FONT></TD>\n",
  //              HT_TABLE_ITEM_FONT_SIZE,text);
  fprintf(fp,"<TD ALIGN=CENTER>%s</TD>\n",text);

}

void htput_table_col_text_left(FILE *fp, const char *text)
{
  //  fprintf(fp,"<TD ALIGN=CENTER><FONT SIZE=%d> %s </FONT></TD>\n",
  //              HT_TABLE_ITEM_FONT_SIZE,text);
  fprintf(fp,"<TD ALIGN=LEFT>%s</TD>\n",text);

}



void htput_table_col_formatted_text(FILE *fp, const char *text)
{
  fprintf(fp,"<TD><pre>\n%s\n</pre></TD>\n",text);

}
 
void htput_table_col_url(FILE *fp, const char *subpath, const char *file, const char *item)
{ 
  //  fprintf(fp,"<TD><A HREF=%c../%s/%s.html%c><FONT SIZE=%d> %s </FONT></A> </TD>\n",
  //	  dq,
  //	  subpath,
  //	  file,
  //	  dq,HT_LIST_ITEM_FONT_SIZE,item);

  fprintf(fp,"<TD ALIGN=LEFT><A HREF=%c../%s/%s.html%c>%s</A> </TD>\n",
	  dq,
	  subpath,
	  file,
	  dq,item);
}

void htput_open_definition_list(FILE *fp)
{
  fprintf(fp,"<DL>\n");
}



void htput_list_item1(FILE *fp, const char *subpath, const char *file, const char *item)
{
  //  fprintf(fp,"<LI><A HREF=%c../%s/%s.html%c><FONT SIZE=%d> %s </FONT></A>\n",
  //	 dq,
  //	 subpath,
  //	 file,
  //	 dq,HT_LIST_ITEM_FONT_SIZE,item);
  fprintf(fp,"<LI><A HREF=%c../%s/%s.html%c>%s</A>\n",
  	 dq,
  	 subpath,
  	 file,
  	 dq,item);
}

void htput_labeled_ref1(FILE *fp, const char *subpath, const char *file, const char *label, const char *item)
{
  fprintf(fp,"<br><h2> %s </h2> <A HREF=%c../%s/%s.html%c> <tt>%s</tt></A> <br>\n",
	  label,
	  dq,
	  subpath,
	  file,
	  dq,
	  item);
}

void htput_labeled_ref1_with_icon(FILE *fp, const char *subpath, const char *file, const char *label, const char *item, const char *icon)
{
  fprintf(fp,"<br><h2><IMG SRC=%c../Icons/%s%c ALIGN=TOP>  %s </h2> <A HREF=%c../%s/%s.html%c>  <tt>%s</tt> </A> \n",
	  dq,
	  icon,dq,
	  label,
	  dq,
	  subpath,
	  file,
	  dq,
	  item);
}


void htput_labeled_text(FILE *fp, const char *label, const char *text)
{
  fprintf(fp,"<br><h2> %s </h2> <tt> %s  </tt><br>\n",label,text);
}

void htput_labeled_text_with_icon(FILE *fp, const char *label, const char *text, const char *icon)
{
  //  fprintf(fp,"<br><h2><IMG SRC=%c../Icons/%s%c ALIGN=TOP> %s </h2> <font size=5><tt> %s </tt></font><br>\n",
  //	  dq,
  //  icon,dq,
  //  label,text);
    fprintf(fp,"<br><h2><IMG SRC=%c../Icons/%s%c ALIGN=TOP> %s </h2> <tt> %s </tt><br>\n",
  	  dq,
    icon,dq,
    label,text);
}

void htput_label_with_icon(FILE *fp, const char *label,  const char *icon)
{
  fprintf(fp,"<br><IMG SRC=%c../Icons/%s%c ALIGN=TOP> <b> %s </b> <br>\n",
	  dq,
	  icon,
	  dq,label);
}


void htput_def_item1(FILE *fp, const char *subpath, const char *file, const char *item, const char *def)
{
  //  fprintf(fp,"<DT><A HREF=%c../%s/%s.html%c><FONT SIZE=%d> %s </A><DD> %s </FONT>\n",
  //	 dq,
  // subpath,
  // file,
  // dq,HT_LIST_ITEM_FONT_SIZE,item,def);
  fprintf(fp,"<DT><A HREF=%c../%s/%s.html%c>%s </A></DT><DD> %s </DD>\n",
	 dq,
	 subpath,
	 file,
	 dq,item,def);
}

void htput_close_definition_list(FILE *fp)
{
  fprintf(fp,"</DL>\n");
}



void htput_open_unordered_list(FILE *fp)
{
  fprintf(fp,"<UL>\n");
}

void htput_close_unordered_list(FILE *fp)
{
  fprintf(fp,"</UL>\n");
}


void htput_pull_down_selection(FILE *fp, const char *label, const char *tag, char **items, int nitems)
{
  int i;
  if (nitems < 0 ) return;
  fprintf(fp,"%s  <SELECT name=%c%s%c> \n",
	 label,
	 dq,tag,dq);

  fprintf(fp,"<OPTION SELECTED> %s\n",items[0]);
  for (i = 1; i < nitems; i++) {
    fprintf(fp,"<OPTION> %s\n", items[i]);
  }
  fprintf(fp,"</SELECT> \n");
}

void htput_scrolled_selection(FILE *fp,const char *label, const char *tag, char **items, 
			      int nitems, int ndisplay)
{
  int i;
  if (nitems < 0 ) return;
  fprintf(fp,"%s  <SELECT NAME=%c%s%c SIZE=%d> \n",
	 label,
	 dq,tag,dq,
	 ndisplay);
  fprintf(fp,"<OPTION SELECTED> %s\n",items[0]);
  for (i = 1; i < nitems; i++) {
    fprintf(fp,"<OPTION> %s\n", items[i]);
  }
  fprintf(fp,"</SELECT> \n");
}


void htput_selection_navigator(FILE *fp, long flags) 
/*
 * Navigation options:
 * 
 * Top Menu             
 * Dictionary Info      
 * Category Groups Menu 
 * Category Menu        
 * Item menu            
 * Data menu
 *
 * This is the navigation line:
 *
 * < Top | Dictionary | Category Groups | Categories | Items | Data>
 */
{
#if defined(HTMLSSI)
  if (flags) fprintf(fp,"\n");
  return;
#else 
  char string[HT_MAX_LINE_LENGTH],url[HT_MAX_URL_LENGTH];
  memset(string,'\0',HT_MAX_LINE_LENGTH);
  sprintf(string,"<br><center>| ");

/*
 * Option: Top Menu 
 */
  if (flags & F_TOP_MENU) {
    memset(url,'\0',HT_MAX_URL_LENGTH);
    sprintf(url,"<A HREF=%c../../index.html%c>Top</A>",
	    dq,dq);
    strcat(string,url);
    strcat(string," | ");
  }


/*
 * Option: Dictionary Info
 */
  if (flags & F_DICT_INFO) {
    memset(url,'\0',HT_MAX_URL_LENGTH);
    sprintf(url,"<A HREF=%c../Index/index.html%c>Dictionary</A>",dq,dq);
    strcat(string,url);
    strcat(string," | ");
  }


/*
 * Option: Category Group Menu 
 */
  if (flags & F_CATEGORY_GROUP_MENU) {
    memset(url,'\0',HT_MAX_URL_LENGTH);
    sprintf(url,"<A HREF=%c../Groups/index.html%c>Category Groups</A>",dq,dq);
    strcat(string,url);
    strcat(string," | ");
  }


/*
 * Option: Category Menu  
 */
  if (flags & F_CATEGORY_MENU) {
    memset(url,'\0',HT_MAX_URL_LENGTH);
    sprintf(url,"<A HREF=%c../Categories/index.html%c>Categories</A>",dq,dq);
    strcat(string,url);
    strcat(string," | ");
  }


/*
 * Option: Item Menu  
 */
  if (flags & F_ITEM_MENU) {
    memset(url,'\0',HT_MAX_URL_LENGTH);
    sprintf(url,"<A HREF=%c../Items/index.html%c>Items</A>",
	    dq,dq);
    strcat(string,url);
    strcat(string," | ");
  }


/*
 * Option: Data Menu  
 */
  if (flags & F_DATA_MENU) {
    memset(url,'\0',HT_MAX_URL_LENGTH);
    sprintf(url,"<A HREF=%c../Data/index.html%c>Data</A>",dq,dq);
    strcat(string,url);
    strcat(string," | ");
  }

  strcat(string," </CENTER>");
  fprintf(fp,"%s\n",string);
#endif
}  



void htput_open_center(FILE *fp)
{
  fprintf(fp,"<CENTER>\n");
}

void htput_close_center(FILE *fp)
{
  fprintf(fp,"</CENTER>\n");
}

void htput_formatted_section(FILE *fp, const char *section, const char *text)
{
  htput_header2_text(fp,section);
  htput_open_preformat(fp);
  fprintf(fp,"\n%s\n",text);
  htput_close_preformat(fp);
}

void htput_formatted_section_with_icon(FILE *fp, const char *section, const char *text, const char *icon)
{
  htput_header2_text_with_icon(fp,section,icon);
  htput_open_preformat(fp);
  fprintf(fp,"\n%s\n",text);
  htput_close_preformat(fp);
}


void htput_example_section(FILE *fp, const string& description, const char *text, int numb)
{
  char cbuf[30];
  memset(cbuf,'\0',30);
  sprintf(cbuf,"Example %d:",numb);
  htput_header3_text(fp,cbuf);

  if (!CifString::IsEmptyValue(description))
  {
/*    fprintf(fp,"<center><i>%s</i> </center>\n",description); */
    htput_open_preformat(fp);
    fprintf(fp,"%s",description.c_str());
    htput_close_preformat(fp);
    htput_line_break(fp);
  }
  htput_open_preformat(fp);
  fprintf(fp,"%s",text);
  htput_close_preformat(fp);
}


void htput_example_section_with_icon(FILE *fp, const string& description, const char *text, int numb, const char *icon)
{
  char cbuf[30];
  memset(cbuf,'\0',30);
  sprintf(cbuf,"Example %d:",numb);
  htput_header3_text_with_icon(fp,cbuf,icon);

  if (!CifString::IsEmptyValue(description)) {
/*     fprintf(fp,"<center><i>%s</i> </center>\n",description); */
    htput_open_preformat(fp);
    fprintf(fp,"%s",description.c_str());
    htput_close_preformat(fp);
    htput_line_break(fp);
  }
  htput_open_preformat(fp);
  fprintf(fp,"%s",text);
  htput_close_preformat(fp);
}


void htput_open_preformat(FILE *fp)
{
  //  fprintf(fp,"<PRE>\n");
  fprintf(fp,"<PRE>");
}

void htput_close_preformat(FILE *fp)
{
  fprintf(fp,"</PRE>\n");
}

void ht_strip_nl(char *string)
{
   int i;
   i = strlen(string);
   if (string[i-1] == '\n') string[i-1] = '\0';
}

